import DBResource from "../../../types/db/DBResource";
import { updatePlanetResources } from "../../db/planets";
import { getAllResources } from "../../db/resources";
import { Planet } from "./PlanetManager";

export type Resource = {
    id: string,
    amount: number,
    lastUpdated: Date,
    perHourMiningRate: number,
    data: DBResource
}

export default class ResourceManager {
    resources: Array<Resource> = [];
    resourcesDB: DBResource[] = [];
    planet: Planet;

    constructor(planet: Planet) {
        this.planet = planet;
    }

    async init(resourceData: { id: string, amount: number, lastUpdated: Date, perHourMiningRate: number }[]) {
        const resources = await getAllResources();

        this.resourcesDB = resources;

        if(resourceData.length === 0) {
            resourceData = [
                {
                    id: "coal",
                    amount: 11,
                    lastUpdated: new Date(),
                    perHourMiningRate: 11
                },
                {
                    id: "iron",
                    amount: 22,
                    lastUpdated: new Date(),
                    perHourMiningRate: 22
                },
                {
                    id: "gold",
                    amount: 33,
                    lastUpdated: new Date(),
                    perHourMiningRate: 33
                },
                {
                    id: "water",
                    amount: 44,
                    lastUpdated: new Date(),
                    perHourMiningRate: 44
                },
                {
                    id: "sulfuric-acid",
                    amount: 55,
                    lastUpdated: new Date(),
                    perHourMiningRate: 55
                },
                {
                    id: "liquid-nitrogen",
                    amount: 66,
                    lastUpdated: new Date(),
                    perHourMiningRate: 66
                },
                {
                    id: "hydrogen",
                    amount: 77,
                    lastUpdated: new Date(),
                    perHourMiningRate: 77
                },
                {
                    id: "oxygen",
                    amount: 88,
                    lastUpdated: new Date(),
                    perHourMiningRate: 88
                },
                {
                    id: "helium-3",
                    amount: 99,
                    lastUpdated: new Date(),
                    perHourMiningRate: 99
                }
            ];

            await updatePlanetResources(this.planet._id, this.resources);
        }
    
        resourceData.forEach(resource => {
            const resFromDB = resources.find(res => res.id === resource.id);
            if(resFromDB) this.resources.push({
                id: resource.id,
                amount: resource.amount,
                lastUpdated: resource.lastUpdated,
                perHourMiningRate: resource.perHourMiningRate,
                data: resFromDB
            })
        });

        return this;
    }

    getResourceById(resId: string) {
        return this.resources.find(res => res.id === resId);
    }

    async calculateCurrentAvailableResources() {
        this.resources.forEach(res => {
            const timeDiff = Math.abs((new Date()).getTime() - res.lastUpdated.getTime());
            const hours = timeDiff / (1000 * 60 * 60);
            const amountToAdd = hours * res.perHourMiningRate;
            res.amount += amountToAdd;
            res.lastUpdated = new Date();
        });

        await updatePlanetResources(this.planet._id, this.resources);

        return this.resources;
    }

    async getDifference(resources: { id: string, amount: number }[]): Promise<Array<Resource>> {
        const currentResources = await this.calculateCurrentAvailableResources();
        const difference: Resource[] = [];

        currentResources.forEach(res => {
            const currentRes = resources.find(r => r.id === res.id);
            if(currentRes) difference.push({
                id: res.id,
                amount: res.amount - currentRes.amount,
                lastUpdated: res.lastUpdated,
                perHourMiningRate: res.perHourMiningRate,
                data: res.data
            });
            else difference.push(res);
        });

        return difference;
    }

    add(resources: Resource[]) {
        for(const res of resources) {
            const resource = this.resources.find(r => r.id === res.id);
            if(resource) resource.amount += res.amount;
            else this.resources.push(res);
        }
    }

    async updateAmount(resources: { id: string, amount: number }[]) {
        await this.calculateCurrentAvailableResources();
        for(const res of resources) {
            const resource = this.resources.find(r => r.id === res.id);
            if(resource) resource.amount += res.amount;
        }
    }

    setAmount(resources: { id: string, amount: number }[]) {
        for(const res of resources) {
            const resource = this.resources.find(r => r.id === res.id);
            if(resource) resource.amount = res.amount;
            else this.resources.push({
                id: res.id,
                amount: res.amount,
                lastUpdated: new Date(),
                perHourMiningRate: 0,
                data: this.resourcesDB.find(r => r.id === res.id)
            });
        }
    }

    async sync() {
        await updatePlanetResources(this.planet._id, this.resources.map(res => {
            return {
                id: res.id,
                amount: res.amount,
                lastUpdated: res.lastUpdated,
                perHourMiningRate: res.perHourMiningRate
            }
        }));
    }
}